==> bgp-parser-api-0.16.16-sources.jar: org/opendaylight/yang/gen/v1/urn/opendaylight/params/xml/ns/yang/bgp/multiprotocol/rev180329/mp/capabilities/add/path/capability/AddressFamiliesBuilder.java <==
package org.opendaylight.yang.gen.v1.urn.opendaylight.params.xml.ns.yang.bgp.multiprotocol.rev180329.mp.capabilities.add.path.capability;
import java.lang.Class;
import java.lang.Object;
import java.lang.Override;
import java.lang.String;
import java.lang.SuppressWarnings;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import javax.annotation.processing.Generated;
import org.opendaylight.yang.gen.v1.urn.opendaylight.params.xml.ns.yang.bgp.multiprotocol.rev180329.BgpAddPathTableType;
import org.opendaylight.yang.gen.v1.urn.opendaylight.params.xml.ns.yang.bgp.multiprotocol.rev180329.BgpTableType;
import org.opendaylight.yang.gen.v1.urn.opendaylight.params.xml.ns.yang.bgp.multiprotocol.rev180329.SendReceive;
import org.opendaylight.yang.gen.v1.urn.opendaylight.params.xml.ns.yang.bgp.types.rev200120.AddressFamily;
import org.opendaylight.yang.gen.v1.urn.opendaylight.params.xml.ns.yang.bgp.types.rev200120.SubsequentAddressFamily;
import org.opendaylight.yangtools.concepts.Builder;
import org.opendaylight.yangtools.yang.binding.AbstractAugmentable;
import org.opendaylight.yangtools.yang.binding.Augmentation;
import org.opendaylight.yangtools.yang.binding.CodeHelpers;
import org.opendaylight.yangtools.yang.binding.DataObject;

/**
 * Class that builds {@link AddressFamilies} instances. Overall design of the class is that of a
 * <a href="https://en.wikipedia.org/wiki/Fluent_interface">fluent interface</a>, where method chaining is used.
 *
 * <p>
 * In general, this class is supposed to be used like this template:
 * <pre>
 *   <code>
 *     AddressFamilies createAddressFamilies(int fooXyzzy, int barBaz) {
 *         return new AddressFamiliesBuilder()
 *             .setFoo(new FooBuilder().setXyzzy(fooXyzzy).build())
 *             .setBar(new BarBuilder().setBaz(barBaz).build())
 *             .build();
 *     }
 *   </code>
 * </pre>
 *
 * <p>
 * This pattern is supported by the immutable nature of AddressFamilies, as instances can be freely passed around without
 * worrying about synchronization issues.
 *
 * <p>
 * As a side note: method chaining results in:
 * <ul>
 *   <li>very efficient Java bytecode, as the method invocation result, in this case the Builder reference, is
 *       on the stack, so further method invocations just need to fill method arguments for the next method
 *       invocation, which is terminated by {@link #build()}, which is then returned from the method</li>
 *   <li>better understanding by humans, as the scope of mutable state (the builder) is kept to a minimum and is
 *       very localized</li>
 *   <li>better optimization opportunities, as the object scope is minimized in terms of invocation (rather than
 *       method) stack, making <a href="https://en.wikipedia.org/wiki/Escape_analysis">escape analysis</a> a lot
 *       easier. Given enough compiler (JIT/AOT) prowess, the cost of th builder object can be completely
 *       eliminated</li>
 * </ul>
 *
 * @see AddressFamilies
 * @see Builder
 *
 */
@Generated("mdsal-binding-generator")
public class AddressFamiliesBuilder implements Builder<AddressFamilies> {

    private Class<? extends AddressFamily> _afi;
    private Class<? extends SubsequentAddressFamily> _safi;
    private SendReceive _sendReceive;


    Map<Class<? extends Augmentation<AddressFamilies>>, Augmentation<AddressFamilies>> augmentation = Collections.emptyMap();

    public AddressFamiliesBuilder() {
    }
    
    
    
    public AddressFamiliesBuilder(BgpAddPathTableType arg) {
        this._sendReceive = arg.getSendReceive();
        this._afi = arg.getAfi();
        this._safi = arg.getSafi();
    }
    public AddressFamiliesBuilder(BgpTableType arg) {
        this._afi = arg.getAfi();
        this._safi = arg.getSafi();
    }

    public AddressFamiliesBuilder(AddressFamilies base) {
        Map<Class<? extends Augmentation<AddressFamilies>>, Augmentation<AddressFamilies>> aug = base.augmentations();
        if (!aug.isEmpty()) {
            this.augmentation = new HashMap<>(aug);
        }
        this._afi = base.getAfi();
        this._safi = base.getSafi();
        this._sendReceive = base.getSendReceive();
    }

    /**
     * Set fields from given grouping argument. Valid argument is instance of one of following types:
     * <ul>
     * <li>BgpTableType</li>
     * <li>BgpAddPathTableType</li>
     * </ul>
     *
     * @param arg grouping object
     * @throws IllegalArgumentException if given argument is none of valid types or has property with incompatible value
    */
    public void fieldsFrom(DataObject arg) {
        boolean isValidArg = false;
        if (arg instanceof BgpTableType) {
            this._afi = ((BgpTableType)arg).getAfi();
            this._safi = ((BgpTableType)arg).getSafi();
            isValidArg = true;
        }
        if (arg instanceof BgpAddPathTableType) {
            this._sendReceive = ((BgpAddPathTableType)arg).getSendReceive();
            isValidArg = true;
        }
        CodeHelpers.validValue(isValidArg, arg, "[BgpTableType, BgpAddPathTableType]");
    }

    public Class<? extends AddressFamily> getAfi() {
        return _afi;
    }
    
    public Class<? extends SubsequentAddressFamily> getSafi() {
        return _safi;
    }
    
    public SendReceive getSendReceive() {
        return _sendReceive;
    }

    @SuppressWarnings({ "unchecked", "checkstyle:methodTypeParameterName"})
    public <E$$ extends Augmentation<AddressFamilies>> E$$ augmentation(Class<E$$> augmentationType) {
        return (E$$) augmentation.get(Objects.requireNonNull(augmentationType));
    }

    
    public AddressFamiliesBuilder setAfi(final Class<? extends AddressFamily> value) {
        this._afi = value;
        return this;
    }
    
    public AddressFamiliesBuilder setSafi(final Class<? extends SubsequentAddressFamily> value) {
        this._safi = value;
        return this;
    }
    
    public AddressFamiliesBuilder setSendReceive(final SendReceive value) {
        this._sendReceive = value;
        return this;
    }
    
    /**
      * Add an augmentation to this builder's product.
      *
      * @param augmentation augmentation to be added
      * @return this builder
      * @throws NullPointerException if {@code augmentation} is null
      */
    public AddressFamiliesBuilder addAugmentation(Augmentation<AddressFamilies> augmentation) {
        Class<? extends Augmentation<AddressFamilies>> augmentationType = augmentation.implementedInterface();
        if (!(this.augmentation instanceof HashMap)) {
            this.augmentation = new HashMap<>();
        }
    
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }
    
    /**
      * Remove an augmentation from this builder's product. If this builder does not track such an augmentation
      * type, this method does nothing.
      *
      * @param augmentationType augmentation type to be removed
      * @return this builder
      */
    public AddressFamiliesBuilder removeAugmentation(Class<? extends Augmentation<AddressFamilies>> augmentationType) {
        if (this.augmentation instanceof HashMap) {
            this.augmentation.remove(augmentationType);
        }
        return this;
    }

    @Override
    public AddressFamilies build() {
        return new AddressFamiliesImpl(this);
    }

    private static final class AddressFamiliesImpl
        extends AbstractAugmentable<AddressFamilies>
        implements AddressFamilies {
    
        private final Class<? extends AddressFamily> _afi;
        private final Class<? extends SubsequentAddressFamily> _safi;
        private final SendReceive _sendReceive;
    
        AddressFamiliesImpl(AddressFamiliesBuilder base) {
            super(base.augmentation);
            this._afi = base.getAfi();
            this._safi = base.getSafi();
            this._sendReceive = base.getSendReceive();
        }
    
        @Override
        public Class<? extends AddressFamily> getAfi() {
            return _afi;
        }
        
        @Override
        public Class<? extends SubsequentAddressFamily> getSafi() {
            return _safi;
        }
        
        @Override
        public SendReceive getSendReceive() {
            return _sendReceive;
        }
    
        private int hash = 0;
        private volatile boolean hashValid = false;
        
        @Override
        public int hashCode() {
            if (hashValid) {
                return hash;
            }
        
            final int result = AddressFamilies.bindingHashCode(this);
            hash = result;
            hashValid = true;
            return result;
        }
    
        @Override
        public boolean equals(Object obj) {
            return AddressFamilies.bindingEquals(this, obj);
        }
    
        @Override
        public String toString() {
            return AddressFamilies.bindingToString(this);
        }
    }
}

==> bgp-parser-api-0.22.8-sources.jar: org/opendaylight/yang/gen/v1/urn/opendaylight/params/xml/ns/yang/bgp/multiprotocol/rev180329/mp/capabilities/add/path/capability/AddressFamiliesBuilder.java <==
package org.opendaylight.yang.gen.v1.urn.opendaylight.params.xml.ns.yang.bgp.multiprotocol.rev180329.mp.capabilities.add.path.capability;
import java.lang.Class;
import java.lang.IllegalArgumentException;
import java.lang.NullPointerException;
import java.lang.Object;
import java.lang.Override;
import java.lang.String;
import java.lang.SuppressWarnings;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import javax.annotation.processing.Generated;
import org.eclipse.jdt.annotation.NonNull;
import org.opendaylight.yang.gen.v1.urn.opendaylight.params.xml.ns.yang.bgp.multiprotocol.rev180329.BgpAddPathTableType;
import org.opendaylight.yang.gen.v1.urn.opendaylight.params.xml.ns.yang.bgp.multiprotocol.rev180329.BgpTableType;
import org.opendaylight.yang.gen.v1.urn.opendaylight.params.xml.ns.yang.bgp.multiprotocol.rev180329.SendReceive;
import org.opendaylight.yang.gen.v1.urn.opendaylight.params.xml.ns.yang.bgp.types.rev200120.AddressFamily;
import org.opendaylight.yang.gen.v1.urn.opendaylight.params.xml.ns.yang.bgp.types.rev200120.SubsequentAddressFamily;
import org.opendaylight.yangtools.binding.Augmentation;
import org.opendaylight.yangtools.binding.Grouping;
import org.opendaylight.yangtools.binding.lib.AbstractAugmentable;
import org.opendaylight.yangtools.binding.lib.CodeHelpers;

/**
 * Class that builds {@link AddressFamilies} instances. Overall design of the class is that of a
 * <a href="https://en.wikipedia.org/wiki/Fluent_interface">fluent interface</a>, where method chaining is used.
 *
 * <p>
 * In general, this class is supposed to be used like this template:
 * <pre>
 *   <code>
 *     AddressFamilies createAddressFamilies(int fooXyzzy, int barBaz) {
 *         return new AddressFamiliesBuilder()
 *             .setFoo(new FooBuilder().setXyzzy(fooXyzzy).build())
 *             .setBar(new BarBuilder().setBaz(barBaz).build())
 *             .build();
 *     }
 *   </code>
 * </pre>
 *
 * <p>
 * This pattern is supported by the immutable nature of AddressFamilies, as instances can be freely passed around without
 * worrying about synchronization issues.
 *
 * <p>
 * As a side note: method chaining results in:
 * <ul>
 *   <li>very efficient Java bytecode, as the method invocation result, in this case the Builder reference, is
 *       on the stack, so further method invocations just need to fill method arguments for the next method
 *       invocation, which is terminated by {@link #build()}, which is then returned from the method</li>
 *   <li>better understanding by humans, as the scope of mutable state (the builder) is kept to a minimum and is
 *       very localized</li>
 *   <li>better optimization opportunities, as the object scope is minimized in terms of invocation (rather than
 *       method) stack, making <a href="https://en.wikipedia.org/wiki/Escape_analysis">escape analysis</a> a lot
 *       easier. Given enough compiler (JIT/AOT) prowess, the cost of th builder object can be completely
 *       eliminated</li>
 * </ul>
 *
 * @see AddressFamilies
 *
 */
@Generated("mdsal-binding-generator")
public class AddressFamiliesBuilder {

    private AddressFamily _afi;
    private SubsequentAddressFamily _safi;
    private SendReceive _sendReceive;


    Map<Class<? extends Augmentation<AddressFamilies>>, Augmentation<AddressFamilies>> augmentation = Map.of();

    /**
     * Construct an empty builder.
     */
    public AddressFamiliesBuilder() {
        // No-op
    }

    
    /**
     * Construct a new builder initialized from specified {@link BgpAddPathTableType}.
     *
     * @param arg BgpAddPathTableType from which the builder should be initialized
     */
    public AddressFamiliesBuilder(BgpAddPathTableType arg) {
        this._sendReceive = arg.getSendReceive();
        this._afi = arg.getAfi();
        this._safi = arg.getSafi();
    }
    
    /**
     * Construct a new builder initialized from specified {@link BgpTableType}.
     *
     * @param arg BgpTableType from which the builder should be initialized
     */
    public AddressFamiliesBuilder(BgpTableType arg) {
        this._afi = arg.getAfi();
        this._safi = arg.getSafi();
    }
    
    

    /**
     * Construct a builder initialized with state from specified {@link AddressFamilies}.
     *
     * @param base AddressFamilies from which the builder should be initialized
     */
    public AddressFamiliesBuilder(final AddressFamilies base) {
        final var aug = base.augmentations();
        if (!aug.isEmpty()) {
            this.augmentation = new HashMap<>(aug);
        }
        this._afi = base.getAfi();
        this._safi = base.getSafi();
        this._sendReceive = base.getSendReceive();
    }

    /**
     * Set fields from given grouping argument. Valid argument is instance of one of following types:
     * <ul>
     *   <li>{@link BgpTableType}</li>
     *   <li>{@link BgpAddPathTableType}</li>
     * </ul>
     *
     * @param arg grouping object
     * @throws IllegalArgumentException if given argument is none of valid types or has property with incompatible value
    */
    public void fieldsFrom(final Grouping arg) {
        boolean isValidArg = false;
        if (arg instanceof BgpTableType castArg) {
            this._afi = castArg.getAfi();
            this._safi = castArg.getSafi();
            isValidArg = true;
        }
        if (arg instanceof BgpAddPathTableType castArg) {
            this._sendReceive = castArg.getSendReceive();
            isValidArg = true;
        }
        CodeHelpers.validValue(isValidArg, arg, "[BgpTableType, BgpAddPathTableType]");
    }


    /**
     * Return current value associated with the property corresponding to {@link AddressFamilies#getAfi()}.
     *
     * @return current value
     */
    public AddressFamily getAfi() {
        return _afi;
    }
    
    /**
     * Return current value associated with the property corresponding to {@link AddressFamilies#getSafi()}.
     *
     * @return current value
     */
    public SubsequentAddressFamily getSafi() {
        return _safi;
    }
    
    /**
     * Return current value associated with the property corresponding to {@link AddressFamilies#getSendReceive()}.
     *
     * @return current value
     */
    public SendReceive getSendReceive() {
        return _sendReceive;
    }

    /**
     * Return the specified augmentation, if it is present in this builder.
     *
     * @param <E$$> augmentation type
     * @param augmentationType augmentation type class
     * @return Augmentation object from this builder, or {@code null} if not present
     * @throws NullPointerException if {@code augmentType} is {@code null}
     */
    @SuppressWarnings({ "unchecked", "checkstyle:methodTypeParameterName"})
    public <E$$ extends Augmentation<AddressFamilies>> E$$ augmentation(Class<E$$> augmentationType) {
        return (E$$) augmentation.get(Objects.requireNonNull(augmentationType));
    }

    
    /**
     * Set the property corresponding to {@link AddressFamilies#getAfi()} to the specified
     * value.
     *
     * @param value desired value
     * @return this builder
     */
    public AddressFamiliesBuilder setAfi(final AddressFamily value) {
        this._afi = value;
        return this;
    }
    
    /**
     * Set the property corresponding to {@link AddressFamilies#getSafi()} to the specified
     * value.
     *
     * @param value desired value
     * @return this builder
     */
    public AddressFamiliesBuilder setSafi(final SubsequentAddressFamily value) {
        this._safi = value;
        return this;
    }
    
    /**
     * Set the property corresponding to {@link AddressFamilies#getSendReceive()} to the specified
     * value.
     *
     * @param value desired value
     * @return this builder
     */
    public AddressFamiliesBuilder setSendReceive(final SendReceive value) {
        this._sendReceive = value;
        return this;
    }
    
    /**
      * Add an augmentation to this builder's product.
      *
      * @param augmentation augmentation to be added
      * @return this builder
      * @throws NullPointerException if {@code augmentation} is null
      */
    public AddressFamiliesBuilder addAugmentation(Augmentation<AddressFamilies> augmentation) {
        if (!(this.augmentation instanceof HashMap)) {
            this.augmentation = new HashMap<>();
        }
    
        this.augmentation.put(augmentation.implementedInterface(), augmentation);
        return this;
    }
    
    /**
      * Remove an augmentation from this builder's product. If this builder does not track such an augmentation
      * type, this method does nothing.
      *
      * @param augmentationType augmentation type to be removed
      * @return this builder
      */
    public AddressFamiliesBuilder removeAugmentation(Class<? extends Augmentation<AddressFamilies>> augmentationType) {
        if (this.augmentation instanceof HashMap) {
            this.augmentation.remove(augmentationType);
        }
        return this;
    }

    /**
     * A new {@link AddressFamilies} instance.
     *
     * @return A new {@link AddressFamilies} instance.
     */
    public @NonNull AddressFamilies build() {
        return new AddressFamiliesImpl(this);
    }

    private static final class AddressFamiliesImpl
        extends AbstractAugmentable<AddressFamilies>
        implements AddressFamilies {
    
        private final AddressFamily _afi;
        private final SubsequentAddressFamily _safi;
        private final SendReceive _sendReceive;
    
        AddressFamiliesImpl(final AddressFamiliesBuilder base) {
            super(base.augmentation);
            this._afi = base.getAfi();
            this._safi = base.getSafi();
            this._sendReceive = base.getSendReceive();
        }
    
        @Override
        public AddressFamily getAfi() {
            return _afi;
        }
        
        @Override
        public SubsequentAddressFamily getSafi() {
            return _safi;
        }
        
        @Override
        public SendReceive getSendReceive() {
            return _sendReceive;
        }
    
        
        
    
        private int hash = 0;
        private volatile boolean hashValid = false;
        
        @Override
        public int hashCode() {
            if (hashValid) {
                return hash;
            }
        
            final int result = AddressFamilies.bindingHashCode(this);
            hash = result;
            hashValid = true;
            return result;
        }
    
        @Override
        public boolean equals(Object obj) {
            return AddressFamilies.bindingEquals(this, obj);
        }
    
        @Override
        public String toString() {
            return AddressFamilies.bindingToString(this);
        }
    }
}
